# Zero Inflated Poisson Regression Model-How to model data with lot of zeroes?

Zero-inflated Poisson regression is used to model count data that has an excess of zero counts. By this I mean that the dependent variable has large number zeros. The theory suggests that the excess zeros are generated by a 2 separate process that can be modeled independently. Thus the zip model has two parts, a Poisson count model and the logit model for predicting excess zeros.

In this section, we’ll learn how to build a regression model for count based datasets in which the dependent variable contains an excess of zero-valued data. We will also learn how to track experiments and features, along with automatic EDA, using VevestaX.


## Table of Contents
1. [Mathematics behind ZIP models](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#mathematics-behind-zip-model)
2. [How to train ZIP models using Python](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#how-to-train-the-zip-model-using-python)
3. [Regression Goal](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#regression-goal)
4. [Performing Train-Test Split](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#performing-train-test-split)
5. [Fitting the ZIP model and Calculating RMSE](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#fitting-the-zip-model-and-calculating-rmse)
6. [Plotting the Actual and Predicted Values](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#plotting-the-actual-and-predicted-values)
7. [Comparing the results with Poisson Distribution](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#comparing-the-results-with-poisson-distribution)
8. [Dumping the model results in Excel Sheet using VevestaX](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#dumping-the-model-results-in-excel-sheet-using-vevestax)
9. [Brief Intro about VevestaX](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/ZIP_tutorial.md#brief-intro-about-vevestax)
10. [References](https://github.com/Vevesta/VevestaX/blob/main/tutorials/ZIP_models/references)

Count based regression models are used where the value of dependent variable is a whole number. Few of the use cases of this model are:

* Number of hits on the website per hour.
* Number of life insurance claims filed per year.
* Number of children a couple has.
* Number of doctor visits per year.
In the real world scenario there are many cases that produce counts which are almost always zero. For example:

* Number of planets discovered each year.
* The number of millionaires living in every single city in the world.

## Mathematics behind Zip Model

When the Poisson regression model is applied to the count outcome data in real world, it is not rare to see the poor model fit indicated by a deviance or Pearson’s chi-square.

The zero-inflated Poisson (ZIP) is an alternative that can be considered here. This model allows for assuming that there are two different types of individuals in the data: (1) Those who have a zero count with a probability of 1 (Always-0 group), and (2) those who have counts predicted by the standard Poisson (Not always-0 group). Observed zero could be from either group, and if the zero is from the Always-0 group, it indicates that the observation is free from the probability of having a positive outcome.

The overall model is a mixture of the probabilities from the two groups, which allows for both the over dispersion and excess zeros that cannot be predicted by the standard Poisson model. For those new to Poisson model, an assumption that must be fulfilled on Poisson distribution is the mean value of data equals to the variance value (or so- called equidispersion). If the variance value is greater than the mean value, it is called overdispersion.

Having a membership of Always-0 group is a binary outcome that can be predicted by logit model.

The probability (𝜓𝑖) that the observation 𝑖 is in Always-0 group is predicted by the characteristic of observation 𝑖, so that can be written as: 𝜓𝑖 = 𝐹(𝑧𝑖 ′𝛾) where 𝑧𝑖 is the vector of covariates and 𝛾 is the vector of coefficients of logit regression.

Then the probability that the observation 𝑖 is in Not always-0 group becomes (1-𝜓𝑖) . For observations in Not always-0 group, their positive count outcome is predicted by the standard Poisson model, so that can be written as:

![image](https://miro.medium.com/1*UfwM8TYXau0LlBPP_Vul0A.png)

where 𝜇𝑖 is the conditional mean.

Then, mixed probabilities for ZIP are expressed as follows:

* Zero counts in Always-0 group

![image](https://miro.medium.com/1*4bxAftTpreDDo_4LXwF6NA.png)

* Zero counts in Not Always-0 group

![image](https://miro.medium.com/1*hcCKyOaO_n44w8K5NkJRaQ.png)

* Non zero counts in Not Always- group

![image](https://miro.medium.com/1*Xk8S81Q0DtgZfr96Im0ZPQ.png)

* Overall

![image](https://miro.medium.com/1*rv-KdfBZy1u8k43pWchHuQ.png)

Here is the practical use case of the same where the state wildlife biologists want to model how many fishes are being caught by the visitors visiting the state park. Some visitors do not fish, but there is no data on whether a person fished or not. Some visitors who did fish did not catch any fish so there are excess zeros in the data because of the people that did not fish.

## How to train the ZIP model using Python?

In our Python tutorial on the ZIP model, we’ll use a data set of camping trips taken by 250 groups of people, the data looks something like this:

![image](https://miro.medium.com/1*NAi1z35BOUFsjDxVFW0iSA.png)

Here is a frequency of Dependent Variable i.e. Fish Count in the dataset,

![image](https://user-images.githubusercontent.com/81908188/168215725-c105d530-760b-47f9-92f7-8b4462714a65.png)

As we can see, there are excess zeroes in this data set. We’ll train a ZIP model on this data set to test this theory and hopefully achieve a better fit than the regular Poisson model.

## Regression Goal

Predict the number of fish caught (FISH_COUNT) by a camping group based on the values of LIVE_BAIT, CAMPER, PERSONS and CHILDREN variables.

## Performing Train-Test Split

![image](https://miro.medium.com/1*w25FKP4s9lFFLuSohgOGSw.png)

## Fitting the ZIP model and Calculating RMSE

![image](https://miro.medium.com/1*OAK6kwsYt6WVusuK8Ooa3A.png)

Here, *endog* is our dependent variable, *exog* is the dataset containing the features or the independent variables and *inflation* is the model for zero inflation which can be either *logit* or *probit*.

## Plotting the Actual and Predicted Values
![image](https://user-images.githubusercontent.com/81908188/168215210-612f407e-bc5d-4e0c-acc9-55a8f1506c57.png)


## Comparing the results with Poisson Distribution

![image](https://miro.medium.com/1*oP1mPzEerL26N1uzIXbl7Q.png)


![image](https://user-images.githubusercontent.com/81908188/168215447-033ce9c9-497b-4972-85f9-847a44257e31.png)

RMSE(ZIP Model) =7.937253933193772

RMSE(Poisson Regression)=8.37973746605465

It is clearly noticeable that the ZIP model has outperformed the Regular Poisson Model on these type of excess zero datasets.

## Dumping the model results in Excel Sheet using VevestaX

![image](https://miro.medium.com/1*dQb3Qcu6SPLI4bIzN55TMA.png)

## Brief Intro about VevestaX

VevestaX is an open source Python package which includes a variety of features that makes the work of a Data Scientist pretty much easier especially when it comes to analyses and getting the insights from the data.

The package can be used to extract the features from the datasets and can track all the variables used in code.

The best part of this package is about its output. The output file of the VevestaX provides us with numerous EDA tools like histograms, performance plots, correlation matrix and much more without writing the actual code for each of them separately.

### How to Use VevestaX?
* Install the package using:
```
pip install vevestaX
```

* Import the library in your kernel as:
```
from vevestaX import vevesta as v
V=v.Experiment( )
```

* To track the feature used:
```
V.ds = df
```
where df is the pandas data frame containing the features.

![image](https://miro.medium.com/1*egUV7Vc-t2gupiG0UU4Myw.png)

* To track features engineered

```
V.fe = df
```

* Finally in order to dump the features and variables used into an excel file and to see the insights what the data carries use:
```
V.dump(techniqueUsed=’ZIP Model’,filename=”ZIP.xlsx”,message=”Zero Inflated Poisson Model was used”,version=1)
```
* In order to check in the experiment and features to [vevesta](https://www.vevesta.com), use the following function:
```
V.commit(techniqueUsed = “Zip Model”, message=”increased accuracy”, version=1, projectId=128, attachmentFlag=True)
```
Note that you need to download access_token.txt into the folder where this jupyter notebook/python script resides.

* Following are the insights we received after dumping the features:

![image](https://miro.medium.com/1*FwqHB7vpNUJek7H4pOb7pw.png)

![image](https://miro.medium.com/1*zX_2WGJvqDF5kH7SGZ6VKA.png)

![image](https://miro.medium.com/1*gP__37J7cghHJhHYB4CmgA.png)

![image](https://miro.medium.com/1*S2sbQ0f_HTrcHp-Y44WoCg.png)

![image](https://miro.medium.com/1*S9RLd7stKvnj-Tlpfixf3Q.png)

![image](https://miro.medium.com/1*TZfmOrmth9psMau5wR2MIA.png)

[*For Source Code Click Here*](https://gist.github.com/sarthakkedia123/7237a61bd9a6697583b1f46b81e43e2c)

This completes our look at the Zero-Inflated Poisson regression model.

## References

1. [Time Series Reasoning](https://timeseriesreasoning.com/contents/zero-inflated-poisson-regression-model/)
2. [alvindayu.com](https://alvindayu.com/yu-the-zero-inflated-poisson-regression-model-time-series-&url=https://alvindayu.com/al-the-zero-inflated-poisson-regression-model-time-series)
3. [usu.edu](https://www.usu.edu/math/jrstevens/biostat/projects2013/rep_ZIP.pdf)
4. [VevestaX GitHub Link](https://github.com/Vevesta/VevestaX)
5. [Article](https://www.vevesta.com/blog/6)

Vevesta is the next generation Portfolio for Machine Learning Project : Save and share machine learning projects. Explore [*Vevesta*](https://www.vevesta.com/?utm_source=ZIP_github) for free. For more such stories, follow us on twitter at [*@vevesta1*](http://twitter.com/vevesta1).

**Author**

Sarthak Kedia

